--------------------------------------------------------------------------------
--- Globals
--------------------------------------------------------------------------------
Global("gwtOpenConfig", View.openConfig)

-- The Window
Global("gwtAuras", View.AurasWindow)

-- Aura Lines
Global("auraLines", { })
Global("auraData", { })
Global("gwtAuraLine", View.AurasWindow.AuraLine)
Global("gwtScrollableContainer", View.AurasWindow.DockPanel.ScrollableContainer)
--------------------------------------------------------------------------------
--- AURA LINES
--------------------------------------------------------------------------------
function InitializeStandardAuraLine()
  gwtAuraLine.widget:Show(false)
  gwtAuraLine.SetImage = function(self, image)
    self.PictureView.widget:SetBackgroundTexture(image)
  end
  gwtAuraLine.SetText = function(self, text)
    self.NameView:setVal("text", text)
  end
end

function CreateAuraLine()
  local new = { }
  new.widget = mainForm:CreateWidgetByDesc(gwtAuraLine.widget:GetWidgetDesc())
  new.PictureView = { }
  new.PictureView.Values = { }
  new.PictureView.widget = new.widget:GetChildChecked("PictureView", false)
  new.NameView = { }
  new.NameView.Values = { }
  new.NameView.widget = new.widget:GetChildChecked("NameView", false)
  new.NameView.setVal = gwtAuraLine.NameView.setVal
  new.Edit = { }
  new.Edit.widget = new.widget:GetChildChecked("Edit", false)
  new.Edit.widget:SetVal("button_label", L["edit"])
  new.Delete = { }
  new.Delete.widget = new.widget:GetChildChecked("Delete", false)
  new.Delete.widget:SetVal("button_label", L["delete"])
  new.SetImage = gwtAuraLine.SetImage
  new.SetText = gwtAuraLine.SetText
  return new
end

function AddAuraLine(image, text)
  local new = CreateAuraLine()
  new:SetImage(image)
  new:SetText(text)
  table.insert(auraLines, new)
end

function UpdateScrollableContainer()
  gwtScrollableContainer.widget:RemoveItems()
  local index = 0
  for k, v in auraData do
    index = index + 1
    if auraLines[index] == nil then
	  auraLines[index] = CreateAuraLine()
	end
	local current = auraLines[index]
	if v.useBuffImage then
	  if v.buff or v.debuff then
	    local buff = SearchImageByBuffName(v.name)
	    if buff then
	      current:SetImage(buff.texture)
	    else
 	      current:SetImage(nil)
	    end
	  elseif v.spellCD then
	    current:SetImage(GetSpellTexture(v.name))
	  end
	else
	  current:SetImage(pictureList[v.pictureIndex].image)
	end
	current:SetText(v.entryname)
	current.Edit.widget:SetName(tostring(k))
	current.Delete.widget:SetName(tostring(k))
	current.name = v.name
	current.pictureIndex = v.pictureIndex
	current.useBuffImage = v.useBuffImage
    gwtScrollableContainer.widget:PushBack(current.widget)
  end
end

function UpdateImagesInScrollableContainer()
  for k, v in auraLines do
    if v.useBuffImage then
	  if v.buff or v.debuff then
	    local buff = SearchImageByBuffName(v.name)
	    if buff then
	      v:SetImage(buff.texture)
	    else
	      v:SetImage(nil)
	    end
	  elseif v.spellCD then
	    v:SetImage(GetSpellTexture(v.name))
	  end
	else
	  v:SetImage(pictureList[v.pictureIndex].image)
	end
  end
end
--------------------------------------------------------------------------------
--- Reactions
--------------------------------------------------------------------------------
onReaction[gwtOpenConfig.reactionName] = function(params)
  if gwtAuras.isVisible or gwtSearchWindow.isVisible or gwtSettings.isVisible then
    gwtSettingsPreview:show(false)
    gwtAuras:show(false)
	gwtSearchWindow:show(false)
	gwtSettings:show(false)
  else
    ShowAurasWindow()
  end
end
--------------------------------------------------------------------------------
--- AURA DATA
--------------------------------------------------------------------------------
function UpdateSettings(settings)
  settings.notinfight = settings.notinfight == nil or settings.notinfight
  settings.infight = settings.infight == nil or settings.infight
  settings.solo = settings.solo == nil or settings.solo
  settings.ingroup = settings.ingroup == nil or settings.ingroup
  settings.inraid = settings.inraid == nil or settings.inraid
  settings.mounted = settings.mounted == nil or settings.mounted
  settings.notmounted = settings.notmounted == nil or settings.notmounted
  settings.hostile = settings.hostile == nil or settings.hostile
  settings.friendly = settings.friendly == nil or settings.friendly
  settings.otherrule = settings.otherrule or common.GetEmptyWString()
  settings.stackminvalue = settings.stackminvalue or 0
  settings.stackmaxvalue = settings.stackmaxvalue or 0
  if settings.showTimer == nil then
    settings.showTimer = true
  end
  if settings.showImage == nil then
    settings.showImage = true
  end
end

function Load()
  auraData = userMods.GetAvatarConfigSection ("PowerAurasData") 
  if auraData == nil then
    auraData = { }
  end
  for k, v in auraData do
    UpdateSettings(v)
  end
end

function Save()
  userMods.SetAvatarConfigSection("PowerAurasData", auraData)
end

function CreateEmptyAuraData()
  local new = { }
  new.entryname = common.GetEmptyWString()
  new.isNew = true
  return new
end
--------------------------------------------------------------------------------
--- REACTIONS
--------------------------------------------------------------------------------
onReaction[View.AurasWindow.DockPanel.ButtomButtons.Add.reactionName] = function(params)
  OpenAuraSettings(CreateEmptyAuraData())
end

onReaction[gwtAuraLine.Edit.reactionName] = function(params)
  if params then
    if params.widget then      
	  local index = tonumber(params.widget:GetName())
	  local data = auraData[index]
	  OpenAuraSettings(data)
	end
  end
end

onReaction[gwtAuraLine.Delete.reactionName] = function(params)
  if params then
    if params.widget then
	  local index = tonumber(params.widget:GetName())
	  table.remove(auraData, index)
	  gwtAuraPanel:UpdateItems() 
	  Save()
	  UpdateScrollableContainer()
	end
  end
end
--------------------------------------------------------------------------------
--- Functions
--------------------------------------------------------------------------------
function ShowAurasWindow()
  if currentAuraData ~= nil then
    if not currentAuraData.isAbort then
	  if currentAuraData.isNew then
	    table.insert(auraData, currentAuraData)
		currentAuraData.isNew = false
	  end
	  Save()
	  UpdateScrollableContainer()	 	  
	end
  end
  UpdateImagesInScrollableContainer()
  gwtAuras:show(true)
  currentAuraData = nil
  gwtAuraPanel:UpdateItems() 
  gwtAuraPanel.widget:SetFade(1.0)
end

--------------------------------------------------------------------------------
--- INITIALIZATION
--------------------------------------------------------------------------------
function InitAuras()  
  gwtAuras:setBackgroundFade(0.8)

  InitializeStandardAuraLine()
  
  Load()
  
  UpdateScrollableContainer()

  gwtAuras:show(false)
  gwtAuras.widget:SetPriority(2000)
end
--------------------------------------------------------------------------------
CallWhenCharacterAvailable(InitAuras)
--------------------------------------------------------------------------------

