--------------------------------------------------------------------------------
--- Globals
--------------------------------------------------------------------------------
Global("gwtAuraPanel", View.AuraPanel)
Global("gwtTimerTemplate", View.TimerTemplate)
Global("isPlayerInCombat", false)
Global("isPlayerInGroup", false)
Global("isPlayerSolo", true)
Global("isPlayerInRaid", false)
Global("isPlayerMounted", false)
Global("buffedUnitIds", { })

--------------------------------------------------------------------------------
--- Functions
--------------------------------------------------------------------------------

function IsIdPlayerOrPlayerPet(unitId)
  local playerId = avatar.GetId()
  return unitId == playerId or unitId == unit. GetActivePet (playerId)
end

function IsIdEnemy(unitId)
  if not unitId then
    return false
  end
  return unit.IsEnemy(unitId)
end

function IsIdFriendly(unitId)
  if not unitId then
    return false
  end
  return unit.IsFriend(unitId)
end

function IsPlayerInCombat()
  local playerId = avatar.GetId()
  return unit.IsInCombat(playerId) or (unit.GetActivePet(playerId) ~= nil and avatar.IsPetInCombat())
end

function IsPlayerInGroup()
  return group.IsPlayerInGroup(avatar.GetId())
end

function IsPlayerSolo()
  return not IsPlayerInGroup() and not IsPlayerInRaid()
end

function IsPlayerInRaid()
  return raid.IsPlayerInAvatarsRaid(object.GetName(avatar.GetId()))
end


function IsPlayerMounted()
  return mount.GetActive() ~= nil
end

function PlayerStates()
  isPlayerInCombat = IsPlayerInCombat()
  isPlayerInGroup = IsPlayerInGroup()
  isPlayerSolo = IsPlayerSolo()
  isPlayerInRaid = IsPlayerInRaid()
  isPlayerMounted = IsPlayerMounted()
end

function CheckExtendedConditions(data, unitId)  
  local buff = data.buffobject
  --if false then
	--LogInfo("Name: ", data.entryname)
	--LogInfo("In Combat: ", data.infight," (",isPlayerInCombat,")")
	--LogInfo("Not In Combat: ", data.notinfight," (",not isPlayerInCombat,")")
	--LogInfo("Solo: ", data.solo," (",isPlayerSolo,")")
	--LogInfo("Raid: ", data.inraid," (",isPlayerInRaid,")")
	--LogInfo("Group: ", data.ingroup," (",isPlayerInGroup,")")
	--LogInfo("Mounted: ", data.mounted," (",isPlayerMounted,")")
	--LogInfo("Not Mounted: ", data.notmounted," (",not isPlayerMounted,")")
	--LogInfo("Hostile: ", data.hostile," (",IsIdEnemy(unitId),")")
	--LogInfo("Friendly: ", data.friendly," (",IsIdFriendly(unitId),")")
	--LogInfo("Use the rule: ", data.useotherrule," (",IsVisibleByName(data.otherrule),")")
	--LogInfo("Casted By Me: ", data.castedbyme," (",buff and (buff.producer and buff.producer.casterId and IsIdPlayerOrPlayerPet(buff.producer.casterId)),")")
	--LogInfo("Stackmin: ", data.stackmin," (",data.stackminvalue, " <= ",(buff and buff.stackCount) or 0,")")	
	--LogInfo("Stackmax: ", data.stackmax," (",data.stackmaxvalue, " >= ", (buff and buff.stackCount) or 0,")")
	--LogInfo("")
  --end
  
  return ((data.infight and isPlayerInCombat) or (data.notinfight and not isPlayerInCombat) or (not data.notinfight and not data.infight))
	     and
	     ((data.solo and isPlayerSolo) or (data.inraid and isPlayerInRaid) or (data.ingroup and isPlayerInGroup) or (not data.solo and not data.ingroup and not data.inraid))
		 and
		 ((data.mounted and isPlayerMounted) or (data.notmounted and not isPlayerMounted) or (not data.mounted and not data.notmounted))
		 and
		 (data.spellCD or (data.hostile and IsIdEnemy(unitId)) or (data.friendly and unitId and IsIdFriendly(unitId)) or (not data.friendly and not data.hostile))
		 and
		 ((not data.useotherrule) or (data.useotherrule and IsVisibleByName(data.otherrule)))		 
end

function GetAuraDataUnitId(data)
  local unitId = nil
  if data.you then
    unitId = avatar.GetId()
  elseif data.yourTarget then
	unitId = avatar.GetTarget()
  elseif data.targetoftarget then
    unitId = avatar.GetTarget()
	if unitId then
	  unitId = unit.GetPrimaryTarget(unitId)
	end
  elseif data.pet then
    unitId = avatar.GetId()
	if unitId then
	  unitId = unit.GetActivePet(unitId)
	end
  end  
  return unitId
end

function CheckAuraData(data, unitId) 
  data.buffobject = nil
  
  if not unitId then
    return nil
  end

  local result = false    
  if data.spellCD then
	local spellbook = avatar.GetSpellBook()
	for i, id in spellbook do 
	  local spellInfo = avatar.GetSpellInfo(id)
	  if (common.CompareWString(spellInfo.name, data.name) == 0) then
	    if spellInfo.cooldownRemainingMs ~= 0 then
	      data.spellobject = spellInfo
		  result = true
		end
		break
  	  end
	end
  elseif data.buff or data.debuff then
    local buffData = GetBuffData(unitId)
    for k, buff in buffData do
      if (data.buff and buff.isPositive) or (data.debuff and not buff.isPositive) then
	    if (common.CompareWString(data.name, buff.name) == 0) then	    
	      UpdateValueByWString(knownBuffs, buff.name, buff)
		  data.buffobject = buff		
	      if (not data.stackmin or (data.stackmin and data.stackminvalue <= ((buff and buff.stackCount) or 0)))
		    and
		    (not data.stackmax or (data.stackmax and data.stackmaxvalue >= ((buff and buff.stackCount) or 0)))
		    and
		    (not data.castedbyme or (data.castedbyme and (buff.producer and buff.producer.casterId and IsIdPlayerOrPlayerPet(buff.producer.casterId)))) then	
		    result = true		  
		    break	
		  end
	    end
  	  end	
  
	  if not result and not data.buffobject and (data.stackmin and data.stackminvalue <= 0) then
        data.buffobject = SearchImageByBuffName(data.name)
	    return true
      end
    end
  end

  return result
end

function IsVisibleByName(name)
  if not name then
    return nil
  end
  for k, v in auraData do 
    if not v.inVisibleByName and v.entryname and common.CompareWString(v.entryname, name) == 0 then
	  v.inVisibleByName = true
	  local result = IsVisible(v)
	  table.remove(v, "inVisibleByName")
	  return result
	end
  end
  return nil
end

function IsVisible(data) 
  local unitId = GetAuraDataUnitId(data)
  local result = CheckAuraData(data, unitId)  
  local extendedConditions = CheckExtendedConditions(data, unitId)
  if result ~= nil then
    if data.show then
	  return result and extendedConditions
    elseif data.hide then
	  return not result and extendedConditions
	end
  end
  return nil
end

gwtAuraPanel.Items = { }
gwtAuraPanel.HideItems = function(self)
  
end
gwtAuraPanel.GetItem = function(self, index)
  local result = self.Items[index]
  if result == nil then
    result = { widget = mainForm:CreateWidgetByDesc(gwtSettingsPreview.widget:GetWidgetDesc()) }
	result.Image = result.widget:GetChildChecked("Image", false)
	result.Cooldown = result.widget:GetChildChecked("Cooldown", false)
	result.Stacksize = result.widget:GetChildChecked("Stacksize", false)
	gwtAuraPanel.widget:AddChild(result.widget)
	self.Items[index] = result
  end
  return result
end
gwtAuraPanel.UpdateItems = function(self)
  self:HideItems()
  local index = 0
  PlayerStates()
  for k, v in auraData do
    index = index + 1	
    local item = self:GetItem(index)	
	v.item = item
	local isInvisible = false
	local isVisible = false
	if v.buff or v.debuff then
	  isVisible = IsVisible(v) == true and v ~= currentAuraData
	  item.widget:Show(isVisible)
	  item.Image:Show(v.showImage)
	  item.Stacksize:Show(v.showImage)
	  item.Cooldown:Show(v.showTimer)
	elseif v.spellCD then
	  local isSpellCD = CheckAuraData(v, true)  
      local isVisible = CheckExtendedConditions(v, true)
	  item.widget:Show(isVisible)
	  if v.show then
	    item.Cooldown:Show(v.showTimer and isSpellCD)
		item.Image:Show(isSpellCD and v.showImage)
	  elseif v.hide then
	    item.Cooldown:Show(v.showTimer and isSpellCD)
		item.Image:Show(not isSpellCD and v.showImage)
	  end	  
	  item.Stacksize:Show(false)
	end
	item.widget:SetPriority(1000+index)	
	if v.showImage then
	  if v.useBuffImage then
	    if v.buff or v.debuff then
	      local buff = v.buffobject or SearchImageByBuffName(v.name)
	      if buff and item.remainingMs ~= buff.remainingMs then
	        item.displayCount = 0
	        item.remainingMs = buff.remainingMs
	      end
	      if buff and buff.texture then	    
	        item.Image:SetBackgroundTexture(buff.texture)
	      else
	        item.Image:SetBackgroundTexture(nil)
		    isInvisible = true
	      end
		elseif v.spellCD then
		  local spellInfo = v.spellobject
		  if spellInfo then
		    if item.remainingMs ~= spellInfo.cooldownRemainingMs then
			  item.displayCount = 0
			  item.remainingMs = spellInfo.cooldownRemainingMs
			end
			if spellInfo.texture then
			  item.Image:SetBackgroundTexture(spellInfo.texture)
			else
			  item.Image:SetBackgroundTexture(nil)
		      isInvisible = true
			end
		  end
		end
	  else
	    item.Image:SetBackgroundTexture(pictureList[v.pictureIndex].image)
	  end	
	  if not isVisible then
	    item.displayCount = 0
	    item.remainingMs = 0
	  end
	end
	if v.widget ~= item.widget then
	  local placement = item.widget:GetPlacementPlain()
	  placement.posX = v.posX
	  placement.posY = v.posY-20
	  placement.sizeX = v.sizeX
	  placement.sizeY = v.sizeY+20
	  item.widget:SetPlacementPlain(placement)	  
	  v.widget = item.widget	  
	end
	if v.opacity == nil then
	  v.opacity = 1.0
	end 
	if isInvisible then
	  item.widget:SetFade(0.0)
	else
	  item.widget:SetFade(v.opacity)
	end	
  end
end
--------------------------------------------------------------------------------
--- EVENTS
--------------------------------------------------------------------------------
function HandleEvent(params)      
  if avatar and avatar.IsExist() then
    ClearBuffData()
    gwtAuraPanel:UpdateItems()
  end
end

function BuffEvents(params) 
  if avatar and avatar.IsExist() then
    if params and params.units then
      for k, v in params.units do
	    if not buffedUnitIds[k] then
		  UpdateKnownBuffs(k)
	      buffedUnitIds[k] = true		
		end
	  end
	end
  end
end

function UpdateKnownBuffsEvent(params)
  for k, v in buffedUnitIds do    
	buffedUnitIds[k] = nil
  end
end

function UpdateDisplayTexts(params)
  for k, v in auraData do
    if v.buffobject and v.item and v.showImage and (v.buff or v.debuff) then
	  local item = v.item
	  item.displayCount = item.displayCount or 0
	  if item.Cooldown then
	    item.Cooldown:SetVal("text", common.FormatFloat((v.buffobject.remainingMs - 100 * item.displayCount) / 1000,"%.2f"))
	  end
	  if item.Stacksize then
		if v.buffobject.stackCount > 1 then
		  item.Stacksize:SetVal("text", common.FormatInt(v.buffobject.stackCount,"%d"))
		else
		  item.Stacksize:SetVal("text", common.GetEmptyWString())
		end
	  end
	  item.displayCount = item.displayCount + 1
	elseif v.spellobject and v.item and v.showImage and v.spellCD then
	  local item = v.item
	  item.displayCount = item.displayCount or 0
	  if item.Cooldown then
		item.Cooldown:SetVal("text", common.FormatFloat((v.spellobject.cooldownRemainingMs - 100 * item.displayCount) / 1000,"%.2f"))
	  end
	  item.Stacksize:SetVal("text", common.GetEmptyWString())
	  item.displayCount = item.displayCount + 1
	end
  end
end

StartTimer(gwtTimerTemplate.widget, 250, HandleEvent)
StartTimer(gwtTimerTemplate.widget, 2000, UpdateKnownBuffsEvent)
StartTimer(gwtTimerTemplate.widget, 100, UpdateDisplayTexts)
onEvent["EVENT_UNIT_BUFF_ADDED"] = BuffEvents
onEvent["EVENT_UNIT_BUFFS_CHANGED"] = BuffEvents
onEvent["EVENT_UNIT_BUFFS_ELEMENT_CHANGED"] = BuffEvents
--------------------------------------------------------------------------------
--- Open Button DND
--------------------------------------------------------------------------------
gwtOpenConfig.initDnD = function(self, wtReacting, fUseCfg, fLockedToScreenArea, Padding )
  if not wtReacting then
    wtReacting = self.widget
  end
  DnD:Init(self.DNDID, wtReacting, self.widget, fUseCfg, fLockedToScreenArea, Padding)
end
gwtOpenConfig.show = function(self, visible)
  if(self.isVisible~=visible) then
    self.isVisible=visible   
  end
  self.widget:Show(visible)
end
gwtOpenConfig.DNDID = 145
gwtOpenConfig:initDnD(nil, true, true)
--------------------------------------------------------------------------------
--- INITIALIZATION
--------------------------------------------------------------------------------
HandleEvent(nil)

gwtSettingsPreview.widget:SetPriority(1200)

View.widget:SetPriority(1500)

